--  Database layout for postgres for SQLJournal
--  Copyright (C) 2005 MJ Ray

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details in the file COPYING.

-- To get destruction commands, try:
--   grep '\-\#' schema.sql | cut -c5- | tac

CREATE TABLE "accounts" (
  "code" varchar(8) unique not null primary key,
  "name" varchar(128) not null,
  "description" text
);

--# DROP TABLE "accounts";

CREATE TABLE "journal" (
  "id" serial primary key,
  "tdate" date default now(),
  "description" text not null,
  "debit" varchar(8) references "accounts"("code"),
  "credit" varchar(8) references "accounts"("code"),
  "amount" numeric(9,2),
  "linked" int references "journal"("id"),
  "note" text,
  "ref" text
);

--# DROP TABLE "journal";

insert into "accounts" values ('A1', 'Cheque Account');
insert into "accounts" values ('AR', 'Accounts Receivable');
insert into "accounts" values ('LP', 'Accounts Payable');
insert into "accounts" values ('L1', 'Liabilities');
insert into "accounts" values ('S1', 'Sales');
insert into "accounts" values ('S7', 'Bank Interest');
insert into "accounts" values ('E1', 'Purchases');
insert into "accounts" values ('E2', 'Travel');
insert into "accounts" values ('E3', 'Stationery');
insert into "accounts" values ('E4', 'Hardware');
insert into "accounts" values ('E5', 'Conferences');
insert into "accounts" values ('E6', 'Marketing');
insert into "accounts" values ('E7', 'Bank Charges');
insert into "accounts" values ('E8', 'Telephony');
insert into "accounts" values ('E9', 'Services');
insert into "accounts" values ('EC1', 'Cost of goods sold');
insert into "accounts" values ('Z1', 'Opening balances');

create view balances as select code, name, ((select coalesce(sum(amount),0) from journal where "debit" = accounts.code)-(select coalesce(sum(amount),0) from journal where "credit" = accounts.code)) as balance from accounts;

--# DROP VIEW "balances";

grant select on accounts,balances,journal to sqlj;
grant insert on journal to sqlj;
grant select,update on journal_id_seq to sqlj;
