#!/usr/bin/mzscheme -qr

;-  SQLJournal - A simple accounting journal
;-  Copyright (C) 2005 MJ Ray

;-  This program is free software; you can redistribute it and/or modify
;-  it under the terms of the GNU General Public License as published by
;-  the Free Software Foundation; either version 2 of the License, or
;-  (at your option) any later version.

;-  This program is distributed in the hope that it will be useful,
;-  but WITHOUT ANY WARRANTY; without even the implied warranty of
;-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;-  GNU General Public License for more details in the file COPYING.

(require (lib "pg.ss" "postgres") (lib "cgi.ss" "net")
  "db.ss"
  "weblib/cgi.ss" "weblib/sxmlout.ss")

(define (join glue buf)
  (if (= (length buf) 1) (car buf) (join glue (cons (string-append (car buf) glue (cadr buf)) (cddr buf)))))

(define (wrap title body)
 `(html
   (head
    (title ,(string-append "SQL-Journal: " title)))
   (body
    (div ((class "nav"))
      (a ((href "./journal")) "Journal") " | "
      (a ((href "./journal?all=t")) "Full Journal") " | "
      (a ((href "./")) "Summary") " | "
      (a ((href "./report")) "Report") " | "
      (a ((href "https://ib.npbs.co.uk/")) "Banking site"))
    (h1 ,title)
    ,@body)))

(define (journal qual)
  (let ((acctsol (option-list "select code,name from accounts order by code desc" db)))
    (wrap "Journal"
     `((form ((action "journal") (method "get"))
       (p "Show only: " (select ((name "ac")) ,@acctsol) "All:" (input ((name "all") (value "t") (type "checkbox"))) (input ((type "submit") (value "Go")))))
       (table
         (tr ,@(map (lambda (l) (list 'th l)) '("Line" "Date" "Description" "To" "From" "Amount" "Note")))
         (tr ((valign "top"))
          (form ((action "./insert") (method "post"))
            (td (input ((type "submit") (value "Add"))))
            (td (input ((name "date") (size "11") (accesskey "n"))))
            (td (input ((name "desc")))
             (br) "Ref" (input ((name "ref")))
             (br) "Note" (input ((name "note"))))
            (td (select ((name "debit")) ,@acctsol))
            (td (select ((name "credit")) ,@acctsol))
            (td (input ((name "amount") (size "8"))))
            (td (input ((type "submit") (value "Add"))))))
         ,@(or (db-map db
             (lambda (v)
               (apply
                 (lambda (id date desc dr cr amt n r)
                   `(tr ((id ,id))
                     (td ,id)
                     (td ,date)
                     (td ,desc (br) "Ref:" ,r)
                     (td ,dr)
                     (td ,cr)
                     (td ,amt)
                     (td ,n)))
                 (map string-copy (vector->list v))))
             (string-append "select id, tdate, j.description, d.name, c.name, amount, note, ref from journal j, accounts c, accounts d where d.code = debit and c.code = credit " qual " order by tdate asc")) (list "")))))))

(display "Content-type: text/html\n\n")
(sxmlout
 (cond
  ((equal? cgi-path "journal")
    (let* ((input (get-bindings/get))
           (ac (extract-bindings 'ac input))
           (all (extract-bindings 'all input)))
      (journal
        (apply string-append
          (cons
            (if (null? all) "and tdate > now() - interval '3 months'" "")
            (if (null? ac)
                '("")
                (list " and (credit = '" (db-quote-string (car ac)) "' or debit='" (db-quote-string (car ac)) "')")))))))
  ((equal? cgi-path "report")
    (wrap "CALPIE Report"
      `((ul
          (li (b "Capital: " ,(string-copy (db-assoc-val db "select (select sum(balance) from balances where code like 'A%')-(select -sum(balance) from balances where code like 'L%')"))))
          (li "Assets: " ,(string-copy (db-assoc-val db "select sum(balance) from balances where code like 'A%'")))
          (li "Liabilities: " ,(string-copy (db-assoc-val db "select -sum(balance) from balances where code like 'L%'")))
          (br)
          (li (b "Profit: " ,(string-copy (db-assoc-val db "select (select sum(amount) from journal where credit like 'S%')-(select sum(amount) from journal where debit like 'E%')"))))
          (li "Income: " ,(string-copy (db-assoc-val db "select sum(amount) from journal where credit like 'S%'")))
          (li "Expenditure: " ,(string-copy (db-assoc-val db "select sum(amount) from journal where debit like 'E%'")))))))
  ((equal? cgi-path "insert")
    (let ((input (get-bindings/post)))
      (db-for-each db list
        (string-append "insert into journal (tdate,description,ref,note,debit,credit,amount) values ('" (join "','" (map (lambda (i) (db-quote-string (extract-binding/single i input))) '(date desc ref note debit credit amount))) "'); select 1"))
      (wrap "Added" (list (list 'p (extract-binding/single 'desc input))))))
  (#t (wrap "Account summary"
   `((table
    (tr (th "Code") (th "Name") (th "Balance"))
    ,@(db-map db (lambda (v) (cons 'tr (map (lambda (l) (list 'td l)) (vector->list v)))) "select code,name,balance from balances order by code desc")))))))

(db-close-port db)
